| C prototype: void FastDrawHLine(unsigned char* plane, short x1, short x2, short y, short mode) __attribute__((__stkparm__));
| Valid values for mode are: A_REVERSE, A_NORMAL, A_XOR, A_REPLACE, A_OR.
|
| This routine draws a horizontal line from (x1;y) to (x2;y).
| This is the version which receives its parameters on the stack.
|
| Many French people often criticize ExtGraph for its slowness. Of course,
| ExtGraph is slower than other libraries, if the equivalent routines in these
| other libraries are written with completely unrolled loops. The comparison
| can't be done in an impartial way, but some of those people do it anyway...
| And they use this kind of arguments to advertise for their own projects (but
| indeed, they don't mention that the comparison is pointless; they don't
| mention either that their projects have serious drawbacks ExtGraph doesn't
| have)...
| Needless to say that these people are not willing to improve it, even if
| they have the skills required to do it (some are more skilled than I am)...
|
| As a result, we are really happy to see that good programmers are not among
| the people mentioned above.
| This routine was contributed to ExtGraph by Julien Richard-Foy
| ('jackiechan'). It is much faster than the previous routine.

.data
.even
__Beginning_FDHL__:
.word 0xFFFF,0x7FFF,0x3FFF,0x1FFF,0x0FFF,0x07FF,0x03FF,0x01FF,0x00FF,0x007F,0x003F,0x001F,0x000F,0x0007,0x0003,0x0001

__End_FDHL__:
.word 0x8000,0xC000,0xE000,0xF000,0xF800,0xFC00,0xFE00,0xFF00,0xFF80,0xFFC0,0xFFE0,0xFFF0,0xFFF8,0xFFFC,0xFFFE,0xFFFF

.globl FastDrawHLine
FastDrawHLine:
    move.w   %d3,-(%sp)                      | d3 mustn't be destroyed.

    movea.l  2+4+0(%sp),%a0
    move.w   2+4+4(%sp),%d0
    move.w   2+4+6(%sp),%d1
    move.w   2+4+8(%sp),%d2
    
    cmp.w    %d0,%d1
    bge.s    __Registers_well_ordered_FDHL__

    exg      %d0,%d1

__Registers_well_ordered_FDHL__:
    move.w   %d2,%d3
    lsl.w    #4,%d2
    sub.w    %d3,%d2
    move.w   %d0,%d3
    lsr.w    #4,%d3
    add.w    %d3,%d2
    add.w    %d2,%d2
    adda.w   %d2,%a0

    lsl.w    #4,%d3
    addi.w   #16,%d3                         | d4 = 8 * (x1/16 + x1/16) + 16.
    move.w   %d1,%d2                         | x2 is stored in d2.
    andi.w   #0xF,%d0

    add.w    %d0,%d0
    move.w   __Beginning_FDHL__(%pc,%d0.w),%d0 | d0 = mask of first pixels.
    andi.w   #0xF,%d1

    add.w    %d1,%d1
    move.w   __End_FDHL__(%pc,%d1.w),%d1     | d1 = mask of last pixels.
    cmp.w    %d3,%d2                         | Line less than 16 pixels long ?
    blt.s    __Less_than_16_pixels_FDHL__
    sub.w    %d3,%d2                         | d2 = x2 - x.
    move.w   2+4+10(%sp),%d3
    beq.s    __Reverse_FDHL__
    cmpi.w   #2,%d3
    beq.s    __Xor_FDHL__

__Normal_FDHL__:
    moveq.l  #32,%d3
    or.w     %d0,(%a0)+
    moveq    #-1,%d0
    sub.w    %d3,%d2
    blt.s    __Normal_16_pixels_FDHL__
__Normal_32_pixels_FDHL__:
    move.l   %d0,(%a0)+
    sub.w    %d3,%d2
    bge.s    __Normal_32_pixels_FDHL__
__Normal_16_pixels_FDHL__:
    cmpi.w   #-16,%d2
    blt.s    __Normal_lessthan16_pixels_FDHL__
    move.w   %d0,(%a0)+
__Normal_lessthan16_pixels_FDHL__:
    or.w     %d1,(%a0)
    move.w   (%sp)+,%d3
    rts

__Reverse_FDHL__:
    moveq.l  #32,%d3
    not.w    %d0
    and.w    %d0,(%a0)+
    moveq    #0,%d0
    sub.w    %d3,%d2
    blt.s    __Reverse_16_pixels_FDHL__
__Reverse_32_pixels_FDHL__:
    move.l   %d0,(%a0)+
    sub.w    %d3,%d2
    bge.s    __Reverse_32_pixels_FDHL__
__Reverse_16_pixels_FDHL__:
    cmpi.w   #-16,%d2
    blt.s    __Reverse_lessthan16_pixels_FDHL__
    move.w   %d0,(%a0)+
__Reverse_lessthan16_pixels_FDHL__:
    not.w    %d1
    and.w    %d1,(%a0)
    move.w   (%sp)+,%d3
    rts

__Xor_FDHL__:
    moveq.l  #32,%d3
    eor.w    %d0,(%a0)+
    sub.w    %d3,%d2
    blt.s    __Xor_16_pixels_FDHL__
__Xor_32_pixels_FDHL__:
    not.l    (%a0)+
    sub.w    %d3,%d2
    bge.s    __Xor_32_pixels_FDHL__
__Xor_16_pixels_FDHL__:
    cmpi.w   #-16,%d2
    blt.s    __Xor_lessthan16_pixels_FDHL__
    not.w    (%a0)+
__Xor_lessthan16_pixels_FDHL__:
    eor.w    %d1,(%a0)
    move.w   (%sp)+,%d3
    rts

__Less_than_16_pixels_FDHL__:
    and.w    %d0,%d1
    move.w   2+4+10(%sp),%d3
    beq.s    __Reverse_l16_FDHL__
    cmpi.w   #2,%d3
    beq.s    __Xor_l16_FDHL__
    or.w     %d1,(%a0)
    move.w   (%sp)+,%d3
    rts
__Reverse_l16_FDHL__:
    not.w    %d1
    and.w    %d1,(%a0)
    move.w   (%sp)+,%d3
    rts
__Xor_l16_FDHL__:
    eor.w    %d1,(%a0)
    move.w   (%sp)+,%d3
    rts
